#define __SYSTEM_SETUP_C__
#include "project.h"

#define MENU_COUNT 10

#define MENU_SAVE      0
#define MENU_CANCEL    1
#define MENU_AVCC      2
#define MENU_BUILTIN   3
#define MENU_WARNING1  4
#define MENU_WARNING2  5
#define MENU_VBUS1     6
#define MENU_VBUS2     7
#define MENU_VBUS_ADJ1 8
#define MENU_VBUS_ADJ2 9

static void flash_backlight( void )
{
	for ( uint8_t i = 0 ;i < 5 ;i++ ) {
		lcdBackLight( OFF );
		WAIT_MS( 30 );
		lcdBackLight( ON );
		WAIT_MS( 30 );
	}
}

/*****************************************************************************
*								EEPϐۑ
* Fvoid
* ߒlFvoid
*****************************************************************************/
void save_eep( void )
{
	uint16_t crc = crc16_CalcRAM( &eep ,sizeof(EEP) );

	eeprom_busy_wait();
	eeprom_write_block( &crc ,(void *)0x0000 ,sizeof(uint16_t) );

	eeprom_busy_wait();
	eeprom_write_block( &eep ,(void *)( 0x0000 + sizeof(uint16_t) ) ,sizeof(EEP) );
}

/*****************************************************************************
*								EEPϐǂݍ
* Fvoid
* ߒlFuint8_t 0:OK !0:CRCsv
*****************************************************************************/
uint8_t load_eep( void )
{
	uint16_t crc = 0;

	eeprom_busy_wait();
	eeprom_read_block( &crc ,(void *)0x0000 ,sizeof(uint16_t) );

	eeprom_busy_wait();
	eeprom_read_block( &eep ,(void *)( 0x0000 + sizeof(uint16_t) ) ,sizeof(EEP) );

	return ( crc == crc16_CalcRAM( &eep ,sizeof(EEP) ) ) ? 0 : 1;
}

/*****************************************************************************
*						VXep[^ݒ
* Fvoid
* ߒlFvoid
*****************************************************************************/
void system_setup( void )
{
	char buf[ 18 ];
	uint8_t mode = 2;
	uint8_t draw = 1;
	uint16_t v = 0U;
	uint16_t vmax = 0U;
	uint16_t vmin = 0U;

	/* ݒ						*/
	dcdc_disable_all();							/* DC/DC o OFF					*/
	keysInit();									/* {^hCo				*/

	lcdCls();
	lcdLocate( 0 ,0 );
	lcdPuts_P( PSTR("  System setup  ") );
	lcdBackLight( ON );

	/* S̃{^nee܂ő҂	*/
	do {
		wdt_reset();
	} while ( ( sw_status( USB_A ) == SW_ON ) || ( sw_status( USB_B ) == SW_ON ) );
	WAIT_MS( 250 );

	do {

		/* e[h̏ʕ\			*/
		lcdCursol( LCD_CURSOL_OFF );
		lcdCls();
		lcdLocate( 0 ,0 );

		dcdc_disable_all();

		if ( mode == MENU_SAVE ) {				/* Save & Exit								*/
			lcdPuts_P( PSTR("Save & Exit") );

		} else if ( mode == MENU_CANCEL ) {		/* Cancel & Exit							*/
			lcdPuts_P( PSTR("Cancel & Exit") );

		} else if ( mode == MENU_AVCC ) {		/* Vref(AVCC)								*/
												/* |CgFAVR #21(AREF) - #22(GND)	*/
			adc_select_vref( 1 );
			v = (uint16_t)( eep.avcc / 1000UL );
			vmin = 4900U;
			vmax = 5100U;
			lcdPuts_P( PSTR("Vref(AVCC)") );

		} else if ( mode == MENU_BUILTIN ) {	/* Vref(Built-in)							*/
												/* |CgFAVR #21(AREF) - #22(GND)	*/
			adc_select_vref( 3 );
			v = (uint16_t)( eep.vref / 1000UL );
			vmin = 1000U;
			vmax = 1200U;
			lcdPuts_P( PSTR("Vref(Built-in)") );

		} else if ( mode == MENU_WARNING1 ) {	/* USB-A Warning I							*/
												/* ݒlȏ̓dƌxv		*/
			v = eep.usb_CurrentWarning[ USB_A ] / 100U;
			vmin = 5U;
			vmax = 32U;
			lcdPuts_P( PSTR("USB-A Warning I") );

		} else if ( mode == MENU_WARNING2 ) {	/* USB-B Warning I							*/
												/* ݒlȏ̓dƌxv		*/
			v = eep.usb_CurrentWarning[ USB_B ] / 100U;
			vmin = 5U;
			vmax = 32U;
			lcdPuts_P( PSTR("USB-B Warning I") );

		} else if ( mode == MENU_VBUS1 ) {		/* USB-A VBUS Voltage						*/
												/* |CgFUSB-A(VBUS) - USB-A(GND)	*/
			dcdc_enable( USB_A );
			v = (uint16_t)( eep.usb_vbus[ USB_A ] / 1000UL );
			vmin = 4800U;
			vmax = 5200U;
			lcdPuts_P( PSTR("USB-A  Voltage") );

		} else if ( mode == MENU_VBUS2 ) {		/* USB-B VBUS Voltage						*/
												/* |CgFUSB-B(VBUS) - USB-B(GND)	*/
			dcdc_enable( USB_B );
			v = (uint16_t)( eep.usb_vbus[ USB_B ] / 1000UL );
			vmin = 4800U;
			vmax = 5200U;
			lcdPuts_P( PSTR("USB-B  Voltage") );

		} else if ( mode == MENU_VBUS_ADJ1 ) {	/* USB-A VBUS Adjust						*/
			v = (uint16_t)eep.vadj[ USB_A ];
			vmin = 0U;
			vmax = 99U;
			lcdPuts_P( PSTR("USB-A  VBUS Adj.") );

		} else if ( mode == MENU_VBUS_ADJ2 ) {	/* USB-B VBUS Adjust						*/
			v = (uint16_t)eep.vadj[ USB_B ];
			vmin = 0U;
			vmax = 99U;
			lcdPuts_P( PSTR("USB-B  VBUS Adj.") );
		}

		/* L[							*/
		keysClear();
		draw = 1;
		while ( 1 ) {
			wdt_reset();
			keysWatchKey();							/* {^Ď			*/

			/* p[^l\					*/
			if ( draw != 0 ) {
				uint8_t m = mode & 0x7f;
				if ( m == MENU_SAVE ) {
					memset( buf ,' ' ,6 );
					buf[ 6 ] = '\0';

				} else if ( m == MENU_CANCEL ) {
					memset( buf ,' ' ,6 );
					buf[ 6 ] = '\0';

				} else if ( m == MENU_AVCC ) {
					sprintf_P( buf ,PSTR("%4umV") ,v );

				} else if ( m == MENU_BUILTIN ) {
					sprintf_P( buf ,PSTR("%4umV") ,v );

				} else if ( m == MENU_WARNING1 ) {
					sprintf_P( buf ,PSTR("%4umA") ,v * 100U );

				} else if ( m == MENU_WARNING2 ) {
					sprintf_P( buf ,PSTR("%4umA") ,v * 100U );

				} else if ( m == MENU_VBUS1 ) {
					sprintf_P( buf ,PSTR("%4umV") ,v );

				} else if ( m == MENU_VBUS2 ) {
					sprintf_P( buf ,PSTR("%4umV") ,v );

				} else if ( m == MENU_VBUS_ADJ1 ) {
					sprintf_P( buf ,PSTR("n=%d.%d") ,v / 10 ,v % 10 );

				} else if ( m == MENU_VBUS_ADJ2 ) {
					sprintf_P( buf ,PSTR("n=%d.%d") ,v / 10 ,v % 10 );

				}

				lcdLocate( 1 ,10 );
				lcdPuts( buf );
				lcdLocate( 1 ,15 );
				draw = 0;
			}

			/* {^						*/
			uint8_t ky = keysInkey();				/* {^Ԏ擾					*/
			if ( ky & KEY_LONGPUSH_START ) {		/* H							*/

				if ( mode & 0x80 ) {				/* ҏWH							*/
					mode &= ~0x80;
					lcdCursol( LCD_CURSOL_OFF );

					if ( mode == MENU_AVCC ) {				/* Vref(AVCC)				*/
						eep.avcc = (uint32_t)v * 1000UL;

					} else if ( mode == MENU_BUILTIN ) {	/* Vref(Built-in)			*/
						eep.vref = (uint32_t)v * 1000UL;

					} else if ( mode == MENU_WARNING1 ) {	/* USB-A Warning I			*/
						eep.usb_CurrentWarning[ USB_A ] = v * 100U;

					} else if ( mode == MENU_WARNING2 ) {	/* USB-B Warning I			*/
						eep.usb_CurrentWarning[ USB_B ] = v * 100U;

					} else if ( mode == MENU_VBUS1 ) {		/* USB-A VBUS Voltage		*/
						eep.usb_vbus[ USB_A ] = (uint32_t)v * 1000UL;

					} else if ( mode == MENU_VBUS2 ) {		/* USB-B VBUS Voltage		*/
						eep.usb_vbus[ USB_B ] = (uint32_t)v * 1000UL;

					} else if ( mode == MENU_VBUS_ADJ1 ) {	/* USB-A VBUS Adjust		*/
						eep.vadj[ USB_A ] = (uint32_t)v;			

					} else if ( mode == MENU_VBUS_ADJ2 ) {	/* USB-B VBUS Adjust		*/
						eep.vadj[ USB_B ] = (uint32_t)v;
					}

				} else {							/* ҏWH						*/
					mode |= 0x80;
					if ( ( mode & 0x7f ) <= 1 ) {	/* Save or Cancel ?					*/
						break;
					}
					lcdCursol( LCD_CURCOL_BLINK );
				}

			} else if ( ky == KEY_SW1 ) {			/* rvPH							*/
				if ( mode & 0x80 ) {				/* ҏWH							*/
					if ( --v < vmin ) v = vmax; 
					draw = 1;

				} else {							/* ҏWH							*/
					if ( --mode == 0xff ) mode = MENU_COUNT - 1;
					break;
				}

			} else if ( ky == KEY_SW2 ) {			/* rvQH							*/
				if ( mode & 0x80 ) {				/* ҏWH							*/
					if ( ++v > vmax ) v = vmin; 
					draw = 1;

				} else {							/* ҏWH							*/
					if ( ++mode >= MENU_COUNT ) mode = 0;
					break;
				}

			}

			cpu_sleep( SLEEP_10MS );				/* 莞ԃX[v		*/
		}

	} while ( !(mode & 0x80) );

	/* p[^[ddoɕۑ			*/
	lcdCls();
	lcdLocate( 0 ,0 );
	lcdPuts_P( PSTR("Edit Parameters") );
	if ( ( mode & 0x7f ) == MENU_SAVE ) {
		save_eep();
		lcdLocate( 1 ,10 );
		lcdPuts_P( PSTR("Saved.") );

	} else {
		lcdLocate( 1 ,6 );
		lcdPuts_P( PSTR("Cancelled.") );
	}
	flash_backlight();

	WAIT_MS( 1000 );
}
